function [u,v,speed] = TCwind_byBattsModel(del_Pmax,Rmax,f,s,x,y)
% construct theoretical tropical cyclone wind field based on Batts typhoon wind field model

% STEPS:

% step1: calculate the maximum gradient wind speed Vgx
% Vgx = K*sqrt(del_Pmax)-0.5*Rmax*f
% K: a coefficient determined empirically as 6.72 (unit: m/s/hPa^2)

% step2: calculate the 10-min maximum wind speed at 10m above the ocean surface
% V(z=10,Rmax) = 0.865Vgx+0.5s
% Vgx: the maximum gradient wind speed calculated in step1

% step3: calculate the 10-min wind speed at 10m above the ocean surface at a distance r from O along the line OM
%  r<Rmax: V(z=10,r) = V(z=10,Rmax)*r/Rmax,
% r>=Rmax: V(z=10,r) = V(z=10,Rmax)*(Rmax/r)^X, X=[0.5,0.7]
% here X is set to 0.6

% step4: calculate the 10-min wind speed at 10m above the ocean surface at a distance r from O on line ON
% V(z=10,r,theta) = V(z=10,r)-0.5*s*(1-cosd(theta))


% Input:
% del_Pmax: pressure drop from periphery of a hurricane to its center (unit:hPa)
% Rmax: radius of the maximum wind speed (unit:km)
% f: Coriolis parameter (unit:1/s)
% s: TC translation speed (unit: m/s)
% theta: the angle between a line ON and the line OM (unit: degree)
% x,y: spatial coordinates

% Output:
% u,v,speed: wind velocity (u,v) and wind speed (speed) based on Batts typhoon wind field model


% Reference: Batts, M. E., Russell, L. R., Simiu, E. (1980) Hurricane wind speeds in the United States.


% calculation process

% step1: Vgx
K=6.72; % unit: m/s/hPa^2
Vgx = K*sqrt(del_Pmax)-0.5*Rmax*f;

% step2: V(z=10,Rmax)
VRmax = 0.865*Vgx+0.5*s;
X=0.6; % set X to 0.6

% step3 & 4: construct theoretical tropical cyclone wind field
[u,v]=uv_Batts(x,y,s,Rmax,VRmax,X);
speed=sqrt(u.*u+v.*v);

end





function [u,v]=uv_Batts(x,y,s,Rmax,VRmax,X)

% construct TC wind field based on V(z=10,r,theta)=V(z=10,r)-0.5*s*(1-cosd(theta)
% coordinate: along-track, cross-track

% Input
% x,y: spatial coordinates
% s: TC translation speed
% Rmax: radius of maximum wind speed
% VRmax: the 10-min　maximum wind speed at 10 m above the ocean surface
% X=[0.5,0.7]

% Output:
% u,v: wind velocity based on Batts typhoon wind field model

% Reference: Batts, M. E., Russell, L. R., Simiu, E. (1980) Hurricane wind speeds in the United States.


% distance
r=sqrt(x.*x+y.*y);

% beta
idx1=r<Rmax;
beta1=r(idx1)/Rmax*10;
idx2=r>=Rmax & r<1.2*Rmax;
beta2=(r(idx2)-Rmax)/(0.2*Rmax)*15+10;
idx3=r>=1.2*Rmax;
beta3=25;
% beat1=5;
% beta2=18;
% beta3=25;

% V(z=10,r)
V_r=nan(size(r));
V_r(idx1)=VRmax.*r(idx1)/Rmax;
V_r(idx2)=VRmax*power(Rmax./r(idx2),X);
V_r(idx3)=VRmax*power(Rmax./r(idx3),X);

% theta
theta_all=nan(size(r));
alpha_all=nan(size(r));
for i=1:length(r(:))
    if (x(i)>=0)
        theta=atand(y(i)/x(i))-(-25); % the angle between a line ON and the line OM
    else
        if (y(i)>0)
            theta=atand(y(i)/x(i))+180-(-25);
        else
            theta=atand(y(i)/x(i))-180-(-25);
        end
    end
    theta_all(i)=theta;
end

% V(z=10,r,theta)
V_theta=V_r-0.5*s.*(1-cosd(theta_all));

alpha_all(idx1)=beta1+theta_all(idx1)+65; % [-180°,180°]
alpha_all(idx2)=beta2+theta_all(idx2)+65; % [-180°,180°]
alpha_all(idx3)=beta3+theta_all(idx3)+65; % [-180°,180°]

u=V_theta .* cosd(alpha_all);
v=V_theta .* sind(alpha_all);

end




